unit Main;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, TabPanel, StdCtrls, Buttons, Picbtn, FileCtrl,
  Icondlg, INIFiles;

type
  TCalThemeForm = class(TForm)
    Panel1: TPanel;
    Image1: TImage;
    Notebook1: TNotebook;
    TabPanel1: TTabPanel;
    Memo1: TMemo;
    Image2: TImage;
    Label1: TLabel;
    Label2: TLabel;
    CloseBtn: TPicBtn;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    SaveINIBtn: TPicBtn;
    Label9: TLabel;
    LibraryEdit: TEdit;
    ThemeBrowseBtn: TPicBtn;
    Label10: TLabel;
    Bevel1: TBevel;
    Label11: TLabel;
    IconEdit: TEdit;
    IconBrowseBtn: TPicBtn;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    ThemeApplyBtn: TPicBtn;
    IconApplyBtn: TPicBtn;
    RestoreDefaultsBtn: TPicBtn;
    KeyCombo: TComboBox;
    IDL_Filenames: TFileListBox;
    Label6: TLabel;
    Label7: TLabel;
    IDS_Wallpaper: TLabel;
    Label17: TLabel;
    IDS_Path: TLabel;
    IDL_Directory: TDirectoryListBox;
    IDC_Drive: TDriveComboBox;
    IDC_Tiled: TCheckBox;
    IDC_Stretch: TCheckBox;
    ClearPaperBtn: TPicBtn;
    ApplyPaperBtn: TPicBtn;
    ScrollBox: TScrollBox;
    IDI_Preview: TImage;
    Bevel2: TBevel;
    IconDialog: TIconDialog;
    FindBtn: TPicBtn;
    FindDialog: TFindDialog;
    Label8: TLabel;
    procedure CloseBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SaveINIBtnClick(Sender: TObject);
    procedure IDL_FilenamesClick(Sender: TObject);
    procedure IDC_StretchClick(Sender: TObject);
    procedure ClearPaperBtnClick(Sender: TObject);
    procedure ApplyPaperBtnClick(Sender: TObject);
    procedure ThemeBrowseBtnClick(Sender: TObject);
    procedure ThemeApplyBtnClick(Sender: TObject);
    procedure RestoreDefaultsBtnClick(Sender: TObject);
    procedure IconBrowseBtnClick(Sender: TObject);
    procedure IconApplyBtnClick(Sender: TObject);
    procedure FindBtnClick(Sender: TObject);
    procedure FindDialogFind(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure Image1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    OldX,
    OldY,
    OldLeft,
    OldTop   : Integer;
    ScreenDC : HDC;
    MoveRect : TRect;
    Moving   : Boolean;
  public
    { Public declarations }
  end;

var
  CalThemeForm: TCalThemeForm;
  CalmiraINILocation : string;
  oldWallpaper: array [0..128] of char;
  oldTiled: integer;

  function ExtractIconFile(const s: string): TFilename;
  function ExtractIconIndex(const s: string): Integer;

implementation

uses ShellAPI, Strings;

{$R *.DFM}

function ExtractIconFile(const s: string): TFilename;
begin
  Result := '';
  Unformat(s, '%s(', [@Result, 79]);
end;

function ExtractIconIndex(const s: string): Integer;
begin
  Result := 0;
  Unformat(s, '%S(%d', [@Result]);
end;

procedure TCalThemeForm.CloseBtnClick(Sender: TObject);
begin
 Close;
end;

procedure TCalThemeForm.FormCreate(Sender: TObject);
begin
 Image2.Picture.Graphic := Application.Icon;
 IconDialog.Filename := Application.EXEName;
 GetProfileString('Desktop', 'Wallpaper', '', oldWallpaper, 128);
 IDS_Wallpaper.Caption := oldWallpaper;
 oldTiled := GetProfileInt('Desktop', 'TileWallpaper', 0);
 IDC_Tiled.Checked := boolean(oldTiled);
end;

procedure TCalThemeForm.FormShow(Sender: TObject);
begin
 CalmiraINILocation := ExtractFilePath(Application.ExeName)+'calmira.ini';
 if FileExists (CalmiraINILocation) then
  memo1.lines.loadfromfile(CalmiraINILocation)
 else begin
  MessageDlg('Unable to find CALMIRA.INI. CalTheme may not be in the correct' +
  ' (Calmira) directory. To prevent problems, CalTheme will now close.',
  mtWarning, [mbOK], 0);
  CalThemeForm.Close;
 end;
end;

procedure TCalThemeForm.SaveINIBtnClick(Sender: TObject);
begin
 CalmiraINILocation := ExtractFilePath(Application.ExeName)+'calmira.ini';
 memo1.lines.savetofile(CalmiraINILocation);
end;

procedure TCalThemeForm.IDL_FilenamesClick(Sender: TObject);
begin
 IDI_Preview.Picture.LoadFromFile(IDL_Filenames.FileName);
 IDC_StretchClick(Sender);
end;

procedure TCalThemeForm.IDC_StretchClick(Sender: TObject);
begin
with ScrollBox do
 begin
  IDI_Preview.Stretch := IDC_Stretch.Checked;
  if (IDI_Preview.Stretch) then
  begin
   IDI_Preview.Width := 125;
   IDI_Preview.Height := 125;
   HorzScrollBar.Visible := False;
   ScrollBox.VertScrollBar.Visible := False;
  end
  else
   begin
    IDI_Preview.AutoSize := True;
    if (IDI_Preview.Width < 125) then
    IDI_Preview.Width := 125
    else
    HorzScrollBar.Visible := True;
    if (IDI_Preview.Height < 125) then
    IDI_Preview.Height := 125
    else
    VertScrollBar.Visible := True;
    HorzScrollBar.Position := (IDI_Preview.Width - Width) div 2;
    VertScrollBar.Position := (IDI_Preview.Height - Height) div 2;
    end;
   end;
end;

procedure TCalThemeForm.ClearPaperBtnClick(Sender: TObject);
var
  fn: array [0..10] of char;
begin
 lStrCpy(fn, '(none)');
 SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, @fn, SPIF_UPDATEINIFILE);
end;

procedure TCalThemeForm.ApplyPaperBtnClick(Sender: TObject);
var
 fn: array [0..128] of char;
 tiled: array [0..1] of char;
begin
 if IDL_Filenames.FileName<>'' then begin
  if IDC_Tiled.Checked then
   lStrCpy(tiled, '1')
  else
   lStrCpy(tiled, '0');
  WriteProfileString('Desktop', 'TileWallPaper', tiled);
  StrPCopy(fn, IDL_Filenames.FileName);
  SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, @fn, SPIF_UPDATEINIFILE)
 end;
end;

procedure TCalThemeForm.ThemeBrowseBtnClick(Sender: TObject);
begin
 KeyCombo.Text := '';
 IconEdit.Text := '';
 with IconDialog do begin
  if execute then
  LibraryEdit.Text := filename;
  end;
end;

procedure TCalThemeForm.ThemeApplyBtnClick(Sender: TObject);
var CalmiraINILocation, ChosenDLL : string;
begin
if LibraryEdit.text > '' then begin
 ChosenDLL := LibraryEdit.text;
 CalmiraINILocation := ExtractFilePath(Application.ExeName)+'calmira.ini';
 { This is why CalTheme must be placed in the same directory as Calmira:
   it relies on itself and CALMIRA.INI being on the same filepath. }
  with TINIFile.Create(CalmiraINILocation) do begin
   try
    WriteString('Icons', '_folder', chosendll+'(0)');
    WriteString('Icons', '_file', chosendll+'(1)');
    WriteString('Icons', '_doc', chosendll+'(2)');
    WriteString('Icons', '_tfile', chosendll+'(3)');
    WriteString('Icons', '_tfolder', chosendll+'(4)');
    WriteString('Icons', '_tprog', chosendll+'(5)');
    WriteString('Icons', '_compute', chosendll+'(6)');
    WriteString('Icons', '_explore', chosendll+'(7)');
    WriteString('Icons', '_windows', chosendll+'(8)');
    WriteString('Icons', '_msdos', chosendll+'(9)');
    WriteString('Icons', '_multi', chosendll+'(10)');
    WriteString('Icons', '_finddlg', chosendll+'(11)');
    WriteString('Icons', '_rundlg', chosendll+'(12)');
    WriteString('Icons', '_hard', chosendll+'(13)');
    WriteString('Icons', '_floppy', chosendll+'(14)');
    WriteString('Icons', '_cdrom', chosendll+'(15)');
    WriteString('Icons', '_ramdisk', chosendll+'(16)');
    WriteString('Icons', '_network', chosendll+'(17)');
    WriteString('Icons', '_emptbin', chosendll+'(18)');
    WriteString('Icons', '_fullbin', chosendll+'(19)');
    WriteString('Icons', '_internt', chosendll+'(20)');
    WriteString('Icons', '_shutdn', chosendll+'(21)');
   finally free;
   Memo1.lines.LoadFromFile(CalmiraINILocation);
   end;
  end;
 end;
end;


procedure TCalThemeForm.RestoreDefaultsBtnClick(Sender: TObject);
begin
 if MessageDlg('Restore default Calmira icons?', mtConfirmation, [mbYes, mbNo], 0)
 = idYes then
 begin
  CalmiraINILocation := ExtractFilePath(Application.ExeName)+'calmira.ini';
   with TINIFile.Create(CalmiraINILocation) do
    try
    WriteString('Icons', '_folder', '');
    WriteString('Icons', '_file', '');
    WriteString('Icons', '_doc', '');
    WriteString('Icons', '_tfile', '');
    WriteString('Icons', '_tfolder', '');
    WriteString('Icons', '_tprog', '');
    WriteString('Icons', '_compute', '');
    WriteString('Icons', '_explore', '');
    WriteString('Icons', '_windows', '');
    WriteString('Icons', '_msdos', '');
    WriteString('Icons', '_multi', '');
    WriteString('Icons', '_finddlg', '');
    WriteString('Icons', '_rundlg', '');
    WriteString('Icons', '_hard', '');
    WriteString('Icons', '_floppy', '');
    WriteString('Icons', '_cdrom', '');
    WriteString('Icons', '_ramdisk', '');
    WriteString('Icons', '_network', '');
    WriteString('Icons', '_emptbin', '');
    WriteString('Icons', '_fullbin', '');
    WriteString('Icons', '_internt', '');
    WriteString('Icons', '_shutdn', '');
    finally free;
   Memo1.lines.LoadFromFile(CalmiraINILocation);
  end;
 end;
end;

procedure TCalThemeForm.IconBrowseBtnClick(Sender: TObject);
begin
LibraryEdit.Text := '';
  with IconDialog do begin
    if IconEdit.Text > '' then Filename := ExtractIconFile(IconEdit.Text);
    IconIndex := ExtractIconIndex(IconEdit.Text);
    if Execute then begin
      IconEdit.Text := Format('%s(%d)', [Filename, IconIndex]);
    end;
  end;
end;

procedure TCalThemeForm.IconApplyBtnClick(Sender: TObject);
begin
CalmiraINILocation := ExtractFilePath(Application.ExeName)+'calmira.ini';
if KeyCombo.Text <> '' then
 if IconEdit.Text <> '' then
  with TINIFile.Create(CalmiraINILocation) do begin
   try WriteString('Icons', KeyCombo.Text, IconEdit.Text);
   finally free;
  end;
  Memo1.lines.LoadFromFile(CalmiraINILocation);
 end;
end;

procedure TCalThemeForm.FindBtnClick(Sender: TObject);
begin
FindDialog.Execute;
end;

procedure TCalThemeForm.FindDialogFind(Sender: TObject);
{ Currently, this dialogue only accepts exact-case matches and forward
  searches. The code to do otherwise is apparently quite complex; I'll
  work on it if/when I have time. }
var
 Buff, P, FT : PChar;
 BuffLen : Word;
begin
 with Sender as TFindDialog do
  begin
   GetMem(FT, Length(FindText) + 1);
   StrPCopy(FT, FindText);
   BuffLen := Memo1.GetTextLen + 1;
   GetMem(Buff, BuffLen);
   Memo1.GetTextBuf(Buff, BuffLen);
   P := Buff + Memo1.SelStart + Memo1.SelLength;
   P := StrPos(P, FT);
   if P = nil then MessageBeep(0)
   else begin
    Memo1.SelStart := P - Buff;
    Memo1.SelLength := Length(FindText);
   end;
  FreeMem(FT, Length(FindText) + 1);
  FreeMem(Buff, BuffLen);
 end;
end;

procedure TCalThemeForm.Image1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then begin
    SetCapture(Panel1.Handle);
    ScreenDC := GetDC(0);
    OldX := X;
    OldY := Y;
    OldLeft := X;
    OldTop := Y;
    MoveRect := BoundsRect;
    DrawFocusRect(ScreenDC,MoveRect);
    Moving := True;
  end;
end;

procedure TCalThemeForm.Image1MouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if Moving then begin
    DrawFocusRect(ScreenDC,MoveRect);
    OldX := X;
    OldY := Y;
    MoveRect := Rect(Left+OldX-OldLeft,Top+OldY-OldTop,
                     Left+Width+OldX-OldLeft,Top+Height+OldY-OldTop);
    DrawFocusRect(ScreenDC,MoveRect);
  end;
end;

procedure TCalThemeForm.Image1MouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then begin
    ReleaseCapture;
    DrawFocusRect(ScreenDC,MoveRect);
    Left := Left+X-OldLeft;
    Top := Top+Y-OldTop;
    ReleaseDC(0,ScreenDC);
    Moving := False;
  end;
end;

end.
